var ng;
dir = [];

//////////////////////////////////////
// Sunburst Directives
//
// This file defines Angular directives 
// that use D3 to draw sunburst charts.
// 

ng = a.directive('budgetSunburstHead', function($compile) {

    function link(scope, element, attr) {

        pscope = scope.$parent;

        var el = element[0];
        $(el).empty();

        var head_div = $("<div />",{"id":"headdiv"});

        var h1 = $("<h1 />").appendTo(head_div);
        var b = $("<b />")
            .text("City of Seattle Budget: [[budget_year]]")
            .appendTo(h1);

        var p = $("<p />", {
            "class":"lead"})
            .text("Each arc in the chart represents an item in the budget. "
                + "The innermost ring is the fund, the middle ring the budget category, "
                + "and the outer ring the budget sub-category or line-item name. "
                + "Hover over an arc to display information in the side box. "
                + "Click on an arc to zoom in on it.")
            .appendTo(head_div);

        var dir = $("div#sunburst_title");
        angular.element(dir).append($compile(head_div)(pscope));

    }
    return {
        restrict: "E",
        link: link,
        scope: {}
    }
});
dir.push(ng);



///////////////////////////////////////////////
// Plain Sunburst Controls

ng = a.directive('budgetSunburstControls', function($compile) {

    function link(scope, element, attr) { 

        pscope = scope.$parent;

        /////////////////////////////////////////
        // Create controls 
        //
        // data has not been loaded yet.
        // start by initializing variables 
        // that don't depend on the data. 

        var el = element[0];

        var ctrl = $("div#sunburst_ctrl");
        ctrl.empty();

        var br = $("<br />").appendTo(ctrl);

        //pscope.countval_btn = "total";
        //pscope.countval_current = "count";
        pscope.countval_btn = "count";
        pscope.countval_current = "total";
        


        // ---------------------------
        // add year picker
        // 2012, 2013, 2014

        var btn_grp = $("<div />", {
                "id" : "yearbtns",
                "class" : "btn-group"
            });

        var a12 = $("<a />",{
                "id" : "btn2012",
                "year" : "2012",
                "changeyear" : "",
                "class":"btn btn-success btn-large btn-year"
            }).text("2012").appendTo(btn_grp);

        var a13 = $("<a />",{
                "id" : "btn2013",
                "year" : "2013",
                "changeyear" : "",
                "class":"btn btn-success btn-large btn-year"
            }).text("2013").appendTo(btn_grp);

        var a14 = $("<a />",{
                "id" : "btn2014",
                "year" : "2014",
                "changeyear" : "",
                "class":"btn btn-success btn-large btn-year"
            }).text("2014").appendTo(btn_grp);

        // to make buttons in this btn group active, 
        // you have to use D3's classed() method 
        // after you add the elements to the document
        // (i.e., after you call $compile)



        // --------------------------
        // add display for details 
        // about current mouseover selection
        
        // assemble the tags, 
        // then compile the html
        // select element of interest with angular.element
        // and append the compiled tags
        //

        var panel = $("<div />", {
                "class" : "panel panel-primary",
                "id" : "selectedPointPanel"
            });

        var panelhead = $("<div />", {
                "class" : "panel-heading"
            }).appendTo(panel);
        
        var h3 = $("<h3 />", {
                "class" : "panel-title"
            }).text("Selected Arc")
            .appendTo(panelhead);



        var panelbody = $("<div />", {
                "class" : "panel-body"
            }).appendTo(panel);

        var maindiv = $("<div />", {
                /*"ng-show" : "selectedPoint"*/
            }).appendTo(panelbody);

        var h = $("<h3 />", {
                "id" : "panelname"
            })
            .html("Name: [[selectedPoint.name]]")
            .appendTo(maindiv);

        var p = $("<p />", {
                "class" : "lead"
            })
            .html("Total: [[selectedPoint.value | currency:'$':0]]")
            .appendTo(maindiv);
            //.html("Total: [[selectedPoint.value | number:0]]")


        // the panel
        //angular.element(chart).prepend($compile(panel)(pscope));
        angular.element(ctrl).prepend($compile(panel)(pscope));

        // year buttons group
        angular.element(ctrl).prepend($compile(btn_grp)(pscope));


        pscope.updateYear = function() {

            // -----------------------------
            // make year button for this 
            // year's budget active 

            d3.selectAll("a.btn-year").classed('active',false);
            var btnlabel = "a#btn"+pscope.budget_year;
            d3.selectAll(btnlabel).classed('active',true);

            console.log("current budget year: "+pscope.budget_year);

        };

        pscope.$watch('budget_year',pscope.updateYear);


    }
    return {
        restrict: "E",
        link: link,
        scope: {}
    }
});
dir.push(ng);




///////////////////////////////////////////////
// action directive: 
// switch between count-based and value-based arc sizing

ng = a.directive("countval", function($compile){
    return function(pscope, element, attrs){
        element.bind("click", function(){

            console.log('in countval directive.');

            if( pscope.countval_btn=="total" ) {
                pscope.countval_btn = "count";
            } else if( pscope.countval_btn=="count" ) {
                pscope.countval_btn = "total";
            }

            if( pscope.countval_current=="total" ) {
                pscope.countval_current = "count";
            } else if( pscope.countval_current=="count" ) {
                pscope.countval_current = "total";
            }

            var key = pscope.countval_current;
            pscope.updateFilter(key)

            pscope.$apply();

        });
    }
});
dir.push(ng);





///////////////////////////////////////////////
// action directive: 
// change to year
// (this is added by the chart drawing method below)

ng = a.directive("changeyear", function($compile){
    return function(pscope, element, attrs){
        element.bind("click", function(){

            // first, update the scope variable holding the 
            // current budget year, and load the corresponding data
            pscope.update_budget_year(attrs['year']);

            // chart controls are redrawn where chart controls are created -
            // a watch was set on the budget_year variable that will 
            // re-draw the chart controls when it changes.

        });
    };
});


///////////////////////////////////////////////
// Plain Sunburst Chart

ng = a.directive('budgetSunburstChart', function($compile) {
    //
    //  buildChart() - build the actual chart, called once
    //      updateChart() - update elements on the chart
    //

    function link(scope, element, attr) {

        scope.$parent.$watch('budgetData',doStuff);

        function doStuff() { 
            if(!scope.$parent.budgetData) { return }
            buildChart(element, scope.$parent);
        }

    };


    function buildChart(element, pscope) { 

        console.log('in buildChart()');

        var data = pscope.budgetData;



        ///////////////////////////////////
        // Create chart svg

        var chart = $("div#sunburst_chart");
        chart.empty();

        // ---------------
        // the chart itself:

        var margin = {
            top:    10, 
            right:  40, 
            bottom: 10, 
            left:   40
        };
        
        var width   = 700 - margin.right - margin.left,
            height  = 700 - margin.top   - margin.bottom;
        
        var radius = Math.min(width, height) / 2;
        
        var x = d3.scale.linear()
            .range([0, 2 * Math.PI]);
        
        var y = d3.scale.sqrt()
            .range([0, radius]);
        
        var svg = d3.select("div#sunburst_chart").append("svg")
            .attr("width", width)
            .attr("height", height)
            .append("g")
            .attr("transform", "translate(" + width / 2 + "," + (height / 2 + 10) + ")");


        // ---------------
        // chart-specific, 
        // data-independent variables:

        var arc = d3.svg.arc()
            .startAngle( function(d) { return Math.max(0, Math.min(2 * Math.PI, x(d.x))); })
            .endAngle(   function(d) { return Math.max(0, Math.min(2 * Math.PI, x(d.x + d.dx))); })
            .innerRadius(function(d) { return Math.max(0, y(d.y)); })
            .outerRadius(function(d) { return Math.max(0, y(d.y + d.dy)); });

        var smallArc_zoom0 = 0.010,
            smallArc_zoom1 = 0.002;

        function stash(d) {
            d.x0 = d.x;
            d.dx0 = d.dx;
        }

        // Tween from zero 
        function arcTweenZero(a, i) {
            var oi = d3.interpolate({x: 0, dx: 0}, a);
            function tween(t) {
                var b = oi(t);
                a.x0  = 0;//b.x;
                a.dx0 = 0;//b.dx;
                return arc(b);
            }
            return tween;
        }


        // When zooming: interpolate the scales.
        function arcTweenZoom(d) {
            var xd = d3.interpolate(x.domain(), [d.x, d.x + d.dx]);
            var yd = d3.interpolate(y.domain(), [d.y, 1]);
            var yr = d3.interpolate(y.range(),  [d.y ? 20 : 0, radius]);
            return function(d, i) {
                return i
                  ? function(t) { return arc(d); }
                  : function(t) { 
                      x.domain(xd(t)); 
                      y.domain(yd(t)).range(yr(t)); 
                      return arc(d); };
            };
        }



        var arcsort = function(a,b) { 
                if(a.total) { 
                    return b.total - a.total;
                } else {
                    return b.value - a.value;
                }
            };


        // default sort method
        var partition = d3.layout.partition()
            .sort(arcsort)
            .value(function(d) { 
                return d.total;
            });

        /*
        var N = 50;
        var color = d3.scale.ordinal()
            .domain([0, N-1])
            .range(d3.range(data.length).map(
                    d3.scale.linear()
                    .domain([0, data.length - 1])
                    .range(["steelblue","pink"])
                    .interpolate(d3.interpolateLab))
            );
        */
        var color = d3.scale.category20b();
        //var color = d3.scale.category20c();

        pscope.updateChart = function() {

            console.log('in updateChart()');

            //////////////////////////////////////////
            // On with the show:
            // draw the damn thing.

            // Keep track of the node that is currently being displayed as the root.
            var node;

            svg.selectAll("path").remove();

            node = data;



            // this is where the magic happens
            // bind the data to the g elements
            var g = svg
                .datum(data)
                .selectAll("g")
                .data(partition.nodes(node))
                .enter();
            
                // !!!!!!!!!!!!!!!!!!
                // note: if you add .append("g") 
                // to the above var g,
                // drawing the arcs will only
                // work the first time you do it,
                // after that they'll disappear
                // and g will be an array of nulls.
                // so don't do this!
                //
                // .append("g");
                // !!!!!!!!!!!!!!!!!!

            // add paths (arcs) to g
            var path = g.append("path")
                .attr("d",arc)
                .style("fill",function(d,i) {
                    if(d.name=="root") {
                        return "#fafafa";
                    } else if(d.depth==1) {
                        return color(d.name);
                    } else if(d.depth==2) {
                        return color(d.parent.name);
                    } else {
                        return color(d.parent.parent.name);
                    }
                })
                .attr("opacity",function(d,i) {
                    if(d.depth==1) {
                        return 0.6;
                    } else if(d.depth==2) {
                        return 0.7;
                    } else if(d.depth==3) {
                        return 0.9;
                    }
                })
                .on("click",click)
                .on('mouseover', function(d){
                    pscope.$apply(function(){
                        pscope.selectedPoint = d;
                    });
                    d3.selectAll('path').classed('active',function(e){
                        var dname = d['name'];
                        var ename = e['name'];
                        return dname==ename;
                    });
                })
                .on('mouseout', function(){
                    d3.selectAll('g.point').classed('active',false);
                })
                .each(stash);
                //.transition()
                //.duration(1400)
                //.attrTween("d",arcTweenZero);



            // add text labels, and set visibility
            var text = g.append("text")
                .attr("transform", function(d) { return "rotate(" + computeTextRotation(d) + ")"; })
                .attr("x", function(d) { return y(d.y); })
                .attr("dx", "6") // margin
                .attr("dy", ".35em") // vertical-align
                .text(function(d) { 
                    if(d.name!="root") {
                        return d.name;
                    }
                })
                .attr("visibility",function(d) {
                    return "hidden";
                    /////if(node.level==1) {
                    /////    if(d.name=="root") {
                    /////        return "hidden";
                    /////    } else {
                    /////        return "visible";
                    /////    }
                    /////} else {
                    /////    return "hidden";
                    /////}
                });



            // animate zooming into partition when clicked 
            function click(d) {

                // this is being screwed up 
                // by the already-existing 
                // arctweenzero transition
                //
                // aaaaaaaaaaaarrrrrrrrrrrrrrrrrrrggggggggggggggggggg

                path.each(stash)
                    .transition()
                    .duration(1400)
                    .attrTween("d",arcTweenZoom(d));

            }


            function computeTextRotation(d) {
                // if our arc takes up more than half a circle,
                // orient the text horizontally.
                if(x(d.dx) > Math.PI) {
                    return 0;
                } else {
                    return (x(d.x + d.dx / 2) - Math.PI / 2) / Math.PI * 180;
                }
            }



        }

        // if you build it, 
        // you must update it.
        pscope.updateChart();

        // chart will be built/updated if user updates data.
        // chart will be updated if user clicks on a point.
        pscope.$watch('selectedPoint',function(){ 
            if(!pscope.selectedPoint) { return };
        });

    }

    return {
        link: link,
        restrict: "E",
        scope: { }
    };
});
dir.push(ng);

